/*
 * Copyright (C) 1998, Center for Geometric Computing, Brown University
 *                 and The Johns Hopkins University
 * 
 * All Rights Reserved
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose other than its incorporation into a  
 * commercial product is hereby granted without fee, provided that the
 * above copyright notice appear in all copies and that both that   
 * copyright notice and this permission notice appear in supporting 
 * documentation, and that the names of Brown University and of The Johns
 * Hopkins University not be used in advertising or publicity pertaining
 * to distribution of the software without specific, written prior
 * permission.
 * 
 * BROWN UNIVERSITY AND THE JOHNS HOPKINS UNIVERSITY DISCLAIM ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY PARTICULAR  
 * PURPOSE. IN NO EVENT SHALL BROWN UNIVERSITY OR THE JOHNS HOPKINS
 * UNIVERSITY BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL 
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

package jdsl.core.api;
import jdsl.simple.api.RankedSequence;
/**
  * Interface for a sequence, as it is described in the
  * Goodrich-Tamassia book.  It extends core.api.PositionalSequence and
  * simple.api.RankedSequence.
  * @see Sequence
  * @see PositionalSequence
  * @see RankedSequence
  * @version JSDL-Teach release 1.0
 * @author Michael T. Goodrich
 * @author Roberto Tamassia
  */

public interface BookSequence extends PositionalSequence,
				      RankedSequence {


  /* *********************  Query Methods  ********************* */

  /** 
    * Return the position with the specified rank.
    * <p> A BoundaryViolationException is thrown if rank < 0 or if
    * rank > n - 1, where n is the size (number of positions) of the
    * sequence.
    * @param rank integer specifying the rank.
    * @return position at the specified rank.
    */
  public Position atRank (int rank)
    throws BoundaryViolationException;
  
  /** 
    * Return the rank of the specified position.  Note that the first
    * position has rank 0, and the last position has rank n - 1, where
    * n is the size (number of positions) of the sequence.
    * @param p position
    * @return rank of the specified position in the sequence.
    */
  public int rankOf (Position p)
    throws InvalidPositionException;

  
  /* *********************  Update Methods  ********************* */
  
  /** 
    * Insert a new position into the sequence at the specified rank,
    * and store the given element into the position.  Return the newly
    * created position.
    * @param rank rank that the new position should have.  For
    * example, if rank = 0, the insertion is done at the beginning,
    * and if rank = n, where n is the old size (number of positions)
    * of the sequence, the insertion is done at the end.
    * @param newElement new element to be inserted.
    * @return newly created position.
    */
  public Position insertAtRank (int rank, Object newElement)
    throws BoundaryViolationException;
  
}
