/*
 * Copyright (C) 1998, Center for Geometric Computing, Brown University
 *                 and The Johns Hopkins University
 * 
 * All Rights Reserved
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose other than its incorporation into a  
 * commercial product is hereby granted without fee, provided that the
 * above copyright notice appear in all copies and that both that   
 * copyright notice and this permission notice appear in supporting 
 * documentation, and that the names of Brown University and of The Johns
 * Hopkins University not be used in advertising or publicity pertaining
 * to distribution of the software without specific, written prior
 * permission.
 * 
 * BROWN UNIVERSITY AND THE JOHNS HOPKINS UNIVERSITY DISCLAIM ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY PARTICULAR  
 * PURPOSE. IN NO EVENT SHALL BROWN UNIVERSITY OR THE JOHNS HOPKINS
 * UNIVERSITY BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL 
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

package jdsl.core.api;
import jdsl.core.ref.AlwaysInvalidLocator;
import java.util.Enumeration;

/**
 * A KeyBasedContainer which provides element lookup based upon search keys. 
 *
 * @version JSDL-Teach release 1.0
 * @author John Kloss
 * @author Mark Handy
 */

public interface Dictionary extends KeyBasedContainer {

  /**
   * Locator returned by find(.) when the key requested is not present
   * in the Dictionary.  Note that dictionaries have
   * special values returned from lookup methods, rather than exceptions,
   * when the lookup fails.
   */
  public static Locator NO_SUCH_KEY = new
    AlwaysInvalidLocator("Dictionary.NO_SUCH_KEY");


  
    /**
     * Returns a Locator mapped to the search key. There is no guarantee that 
     * the mapping of this Locator to <code>key</code> is unique-- there may 
     * be many Locators mapped to the same key depending on the particular 
     * implementation of this Dictionary. Furthermore, there may be <I>no</I>
     * such element mapped to <code>key</code> in which case the Locator 
     * returned will be NO_SUCH_KEY, which is invalid. That is, it
     * does not reference an element in any Container and cannot be used.
     *
     * @param              key  The key used to search for an element in this
     *                     Container.
     *
     * @return             A Locator mapped to <code>key</code>. <B>Note:</B> 
     *                     this Locator will be NO_SUCH_KEY, an
     *                     invalid locator, if the key is not found.
     *
     * @exception          InvalidKeyException  If <code>key</code> is not
     *                     of a type accepted by this Container (For example:
     *                     The key is not comparable by this Container).
     */
    Locator find(Object key)
	throws InvalidKeyException;

    /**
     * Returns an Enumeration of Locators mapped to the search key. This
     * method is guaranteed to return all elements associated with 
     * <code>key</code>. If there are no such elements then the Enumeration 
     * is empty.  There is no guaranteed of order of these
     * <code>Locators</code>.
     *
     * @param              key  The key used to search for an element within
     *                     this Container.
     *
     * @return             An Enumeration of Locators mapped to <code>key
     *                     </code>. <B>Note:</B> the Enumeration will be empty 
     *                     if no such elements exist in the container.
     *
     * @exception          InvalidKeyException  If <code>key</code> is not
     *                     of a type accepted by this Container (For example:
     *                     The key is not comparable by this Container).
     */
    Enumeration findAll(Object key)
	throws InvalidKeyException;
}
