/*
 * Copyright (C) 1998, Center for Geometric Computing, Brown University
 *                 and The Johns Hopkins University
 * 
 * All Rights Reserved
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose other than its incorporation into a  
 * commercial product is hereby granted without fee, provided that the
 * above copyright notice appear in all copies and that both that   
 * copyright notice and this permission notice appear in supporting 
 * documentation, and that the names of Brown University and of The Johns
 * Hopkins University not be used in advertising or publicity pertaining
 * to distribution of the software without specific, written prior
 * permission.
 * 
 * BROWN UNIVERSITY AND THE JOHNS HOPKINS UNIVERSITY DISCLAIM ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY PARTICULAR  
 * PURPOSE. IN NO EVENT SHALL BROWN UNIVERSITY OR THE JOHNS HOPKINS
 * UNIVERSITY BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL 
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

package jdsl.core.api;

/**
 * Interface for a binary tree, or a data structure that can be viewed
 * as a binary tree.  No methods that modify the tree are provided;
 * the tree can only be viewed.
 * 
 * For a binary tree, methods Container.elements() and
 * PositionalContainer.positions() are guaranteed to return
 * enumerations in preorder order. 
 *   
 * @author Mark Handy
 * @version JSDL-Teach release 1.0
 */

public interface InspectableBinaryTree extends InspectableTree {


    
     /* *********************  Accessor Methods  ********************* */

    
/** 
 * Gets the left child of p
 * @param p Any node of the tree
 * @return left child of the given node
 * @exception BoundaryViolationException If Position p is external
 * @exception InvalidPositionException If Position p is null or incompatible
 * with this container
 * @exception InvalidContainerException If this container is invalid.
 */
  public Position leftChild (Position p)
    throws InvalidPositionException, BoundaryViolationException,InvalidContainerException;


/** 
 * Gets the right child of p.
 * @param p Any node of the tree
 * @return right child of the given node
 * @exception BoundaryViolationException If Position p is external
 * @exception InvalidPositionException If Position p is null or incompatible
 * with this container
 * @exception InvalidContainerException If this container is invalid.
 */
  public Position rightChild (Position p)
    throws InvalidPositionException, BoundaryViolationException,InvalidContainerException;


/** 
 * Gets the sibling <code>Position</code> of the 
 * <code>Position</code> passed in.
 * @param otherChild Any node of the tree
 * @return sibling of the given node (the other child of this node's parent)
 * @exception BoundaryViolationException If Position p is the root
 * @exception InvalidPositionException If Position p is null or incompatible
 * with this container
 * @exception InvalidContainerException If this container is invalid.
 */
  public Position sibling (Position p)
    throws InvalidPositionException, BoundaryViolationException,InvalidContainerException;
  
}
