/*
 * Copyright (C) 1998, Center for Geometric Computing, Brown University
 *                 and The Johns Hopkins University
 * 
 * All Rights Reserved
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose other than its incorporation into a  
 * commercial product is hereby granted without fee, provided that the
 * above copyright notice appear in all copies and that both that   
 * copyright notice and this permission notice appear in supporting 
 * documentation, and that the names of Brown University and of The Johns
 * Hopkins University not be used in advertising or publicity pertaining
 * to distribution of the software without specific, written prior
 * permission.
 * 
 * BROWN UNIVERSITY AND THE JOHNS HOPKINS UNIVERSITY DISCLAIM ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY PARTICULAR  
 * PURPOSE. IN NO EVENT SHALL BROWN UNIVERSITY OR THE JOHNS HOPKINS
 * UNIVERSITY BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL 
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

package jdsl.core.api;

/**

 * Interface for a position.  A position formalizes the intuitive
 * notion of the "place" where an element is stored in a positional
 * container (sequence, tree, graph).  A node of a tree and a vertex
 * of a graph are examples of positions.

 * @see PositionalContainer
 * @see InvalidPositionException

 * @version JSDL-Teach release 1.0

 * @author Michael T. Goodrich
 * @author Mark Handy
 * @author Roberto Tamassia

 */

public interface Position {

/** 
 * Return the element stored at this position.
 * <p> An InvalidPositionException is thrown if the position is
 * invalid because it was removed from its container.
 * @return element stored at this position
 */
  public Object element()
    throws InvalidPositionException;

/** 
 * Return the container holding this position.
 * <p> An InvalidPositionException is thrown if the position is
 * invalid because it was removed from its container.
 * @return container holding this position.
 */
  public Container container()
    throws InvalidPositionException;

}
