/*
 * Copyright (C) 1998, Center for Geometric Computing, Brown University
 *                 and The Johns Hopkins University
 * 
 * All Rights Reserved
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose other than its incorporation into a  
 * commercial product is hereby granted without fee, provided that the
 * above copyright notice appear in all copies and that both that   
 * copyright notice and this permission notice appear in supporting 
 * documentation, and that the names of Brown University and of The Johns
 * Hopkins University not be used in advertising or publicity pertaining
 * to distribution of the software without specific, written prior
 * permission.
 * 
 * BROWN UNIVERSITY AND THE JOHNS HOPKINS UNIVERSITY DISCLAIM ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY PARTICULAR  
 * PURPOSE. IN NO EVENT SHALL BROWN UNIVERSITY OR THE JOHNS HOPKINS
 * UNIVERSITY BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL 
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

package jdsl.core.api;

/**
  *
  * Adds the ability of rotation to a BinaryTree
  *
  * @version JSDL-Teach release 1.0
  */

public interface RestructurableBinaryTree extends BinaryTree {

  

/* *********************  Update Method  ********************* */

    
/** 
 * Applies a rotation to the given position, its parent,
 * and its grandparent.  Their in-order order will be preserved, along
 * with the in-order order of all their children.
 *
 * @param grandchild the deepest of the three positions of this
 * restructuring
 * @return The position now at the root of the newly rotated subtree
 * @exception BoundaryViolationException If Position grandchild 
 * is not the grandchild of any Position -- that is, if Position
 * grandchild is a child of the root, or the root itself.
 * @exception InvalidPositionException In addition to all the usual
 * cases when an IPE is thrown, one is thrown when restructure(.)
 * is called on an external node.
 */
  public Position restructure (Position grandchild)
      throws BoundaryViolationException, InvalidPositionException;

}
