/*
 * Copyright (C) 1998, Center for Geometric Computing, Brown University
 *                 and The Johns Hopkins University
 * 
 * All Rights Reserved
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose other than its incorporation into a  
 * commercial product is hereby granted without fee, provided that the
 * above copyright notice appear in all copies and that both that   
 * copyright notice and this permission notice appear in supporting 
 * documentation, and that the names of Brown University and of The Johns
 * Hopkins University not be used in advertising or publicity pertaining
 * to distribution of the software without specific, written prior
 * permission.
 * 
 * BROWN UNIVERSITY AND THE JOHNS HOPKINS UNIVERSITY DISCLAIM ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY PARTICULAR  
 * PURPOSE. IN NO EVENT SHALL BROWN UNIVERSITY OR THE JOHNS HOPKINS
 * UNIVERSITY BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL 
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

package jdsl.core.api;
import java.util.Enumeration;

/**
  * Seperates the methods cut, link, and replaceSubtree out of BinaryTree
  * so that faster implementations of BinaryTree can be created.
  *
 * @author Mike Boilen
  * @version JSDL-Teach release 1.0
  */

public interface Tree extends InspectableTree {


    /* *********************  Update Methods  ********************* */

  
/** 
 * This tree is cut above the position passed in, and that position is replaced
 * with an external node with a null element. The subtree cut off at
 * that point is returned to the user as a new Tree.
 *
 * @param root The position above which to make the cut; will be the
 * root of the tree passed back 
 * @return the subtree cut off of the tree.
 * @exception InvalidPositionException if position is null or incompatible
 * with this container
 */ 
  public Tree cut (Position root) throws InvalidPositionException;


/** 
 * Position mustBeExternal is removed from the tree, and its locator is
 * made uncontained.  The root of newSubtree, with all its children
 * and associated locators and elements, is attached as the new child
 * of mustBeExternal's parent.  newSubtree becomes invalid.
 *
 * @param root The position to insert the new Subtree.
 * @param newSubtree The subtree to insert at the position.
 * @exception InvalidPositionException if position is null or incompatible
 * with this container
 */  
  public void link (Position mustBeExternal, Tree newSubtree) throws 
    InvalidPositionException;

  
/** 
 * Swap out the tree rooted at subtreeRoot and replace it with the
 * newSubtree passed in.  The subtree removed is returned as a new
 * Tree.  The Tree passed in becomes invalid.
 *
 * @param subtreeRoot A position in the DynamicBinaryTree on which
 * the method is called
 * @param newSubtree The subtree which is to be inserted in the place
 * subTreeRoot currently occupies. 
 * @return A new tree, with subtreeRoot as its root
 * @exception InvalidPositionException if position is null or incompatible
 * with this container
 */ 
  public Tree replaceSubtree (Position subtreeRoot,
			      Tree newSubtree) throws
    InvalidPositionException;
}
