/*
 * Copyright (C) 1998, Center for Geometric Computing, Brown University
 *                 and The Johns Hopkins University
 * 
 * All Rights Reserved
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose other than its incorporation into a  
 * commercial product is hereby granted without fee, provided that the
 * above copyright notice appear in all copies and that both that   
 * copyright notice and this permission notice appear in supporting 
 * documentation, and that the names of Brown University and of The Johns
 * Hopkins University not be used in advertising or publicity pertaining
 * to distribution of the software without specific, written prior
 * permission.
 * 
 * BROWN UNIVERSITY AND THE JOHNS HOPKINS UNIVERSITY DISCLAIM ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY PARTICULAR  
 * PURPOSE. IN NO EVENT SHALL BROWN UNIVERSITY OR THE JOHNS HOPKINS
 * UNIVERSITY BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL 
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

package jdsl.simple.api;

/** 
 * Interface for a deque, or double-ended queue.  <p> A deque is a
 * collection of linearly arranged elements that are accessed,
 * inserted, and removed at the beginning or at the end.
 *
 * @author Michael T. Goodrich
 * @author Mark Handy
 * @author Roberto Tamassia
 * @version JSDL-Teach release 1.0
 * @see DequeEmptyException
 * @see Queue
 * @see Stack

 */

public interface Deque {

  // accessor methods

/** 
 * Gets the first element (without modifying the deque).
 *
 * @return first element in the deque.
 * @exception DequeEmptyException if the deque is empty
 */
  public Object first()  
    throws DequeEmptyException; 

/** 
 * Gets the last element (without modifying the deque).
 *
 * @return last element in the deque.
 * @exception DequeEmptyException if the deque is empty
 */
  public Object last()  
    throws DequeEmptyException;  

/** 
 * Tests if this deque is empty
 *
 * @return <code>true</code> if the deque is empty, <code>false</code>
 * otherwise.
 */
  public boolean isEmpty();  


/** 
 * Gets the number of elements. 
 *
 * @return  number of elements in the deque.
 */
  public int size();  

  // update methods

/** 
 * Insert an element at the beginning.
 *
 * @param element new element to be inserted.
 */
  public void insertFirst (Object element);  

/** 
 * Insert an element at the end.
 *
 * @param element new element to be inserted.
 */
  public void insertLast (Object element);  

/** 
 * Remove the element at the beginning.
 *
 * @return element removed.
 * @exception DequeEmptyException if the deque is empty
 */
  public Object removeFirst()  
    throws DequeEmptyException; 

/** 
 * Remove the element at the end.
 *
 * @return element removed.
 * @exception DequeEmptyException if the deque is empty
 */
  public Object removeLast()  
    throws DequeEmptyException;  

}
